@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -52,9 +44,10 @@
 // sdk
 #include <QtDesigner/QDesignerFormEditorInterface>
 #include <QtDesigner/QDesignerDnDItemInterface>
-#include <QtDesigner/QDesignerCustomWidgetInterface>
 #include <QtDesigner/QDesignerSettingsInterface>
 
+#include <QtUiPlugin/QDesignerCustomWidgetInterface>
+
 #include <QtWidgets/QHeaderView>
 #include <QtWidgets/QApplication>
 #include <QtWidgets/QTreeWidgetItem>
@@ -79,7 +72,6 @@ static const char *defaultTypeValueC = "default";
 static const char *customValueC = "custom";
 static const char *iconPrefixC = "__qt_icon__";
 static const char *scratchPadValueC = "scratchpad";
-static const char *qtLogoC = "qtlogo.png";
 static const char *invisibleNameC = "[invisible]";
 
 enum TopLevelRole  { NORMAL_ITEM, SCRATCHPAD_ITEM, CUSTOM_ITEM };
@@ -115,14 +107,14 @@ WidgetBoxTreeWidget::WidgetBoxTreeWidget(QDesignerFormEditorInterface *core, QWi
 
     setItemDelegate(new SheetDelegate(this, this));
 
-    connect(this, SIGNAL(itemPressed(QTreeWidgetItem*,int)),
-            this, SLOT(handleMousePress(QTreeWidgetItem*)));
+    connect(this, &QTreeWidget::itemPressed,
+            this, &WidgetBoxTreeWidget::handleMousePress);
 }
 
 QIcon WidgetBoxTreeWidget::iconForWidget(QString iconName) const
 {
     if (iconName.isEmpty())
-        iconName = QLatin1String(qtLogoC);
+        return qdesigner_internal::qtLogoIcon();
 
     if (iconName.startsWith(QLatin1String(iconPrefixC))) {
         const IconCache::const_iterator it = m_pluginIcons.constFind(iconName);
@@ -142,6 +134,10 @@ WidgetBoxCategoryListView *WidgetBoxTreeWidget::categoryViewAt(int idx) const
     return rc;
 }
 
+static const char widgetBoxSettingsGroupC[] = "WidgetBox";
+static const char widgetBoxExpandedKeyC[] = "Closed categories";
+static const char widgetBoxViewModeKeyC[] = "View mode";
+
 void WidgetBoxTreeWidget::saveExpandedState() const
 {
     QStringList closedCategories;
@@ -153,9 +149,9 @@ void WidgetBoxTreeWidget::saveExpandedState() const
         }
     }
     QDesignerSettingsInterface *settings = m_core->settingsManager();
-    settings->beginGroup(QLatin1String(widgetBoxRootElementC));
-    settings->setValue(QStringLiteral("Closed categories"), closedCategories);
-    settings->setValue(QStringLiteral("View mode"), m_iconMode);
+    settings->beginGroup(QLatin1String(widgetBoxSettingsGroupC));
+    settings->setValue(QLatin1String(widgetBoxExpandedKeyC), closedCategories);
+    settings->setValue(QLatin1String(widgetBoxViewModeKeyC), m_iconMode);
     settings->endGroup();
 }
 
@@ -163,9 +159,10 @@ void  WidgetBoxTreeWidget::restoreExpandedState()
 {
     typedef QSet<QString> StringSet;
     QDesignerSettingsInterface *settings = m_core->settingsManager();
-    m_iconMode = settings->value(QStringLiteral("WidgetBox/View mode")).toBool();
+    const QString groupKey = QLatin1String(widgetBoxSettingsGroupC) + QLatin1Char('/');
+    m_iconMode = settings->value(groupKey + QLatin1String(widgetBoxViewModeKeyC)).toBool();
     updateViewMode();
-    const StringSet closedCategories = settings->value(QStringLiteral("WidgetBox/Closed categories"), QStringList()).toStringList().toSet();
+    const StringSet closedCategories = settings->value(groupKey + QLatin1String(widgetBoxExpandedKeyC), QStringList()).toStringList().toSet();
     expandAll();
     if (closedCategories.empty())
         return;
@@ -256,10 +253,14 @@ WidgetBoxCategoryListView *WidgetBoxTreeWidget::addCategoryView(QTreeWidgetItem
     embed_item->setFlags(Qt::ItemIsEnabled);
     WidgetBoxCategoryListView *categoryView = new WidgetBoxCategoryListView(m_core, this);
     categoryView->setViewMode(iconMode ? QListView::IconMode : QListView::ListMode);
-    connect(categoryView, SIGNAL(scratchPadChanged()), this, SLOT(slotSave()));
-    connect(categoryView, SIGNAL(pressed(QString,QString,QPoint)), this, SIGNAL(pressed(QString,QString,QPoint)));
-    connect(categoryView, SIGNAL(itemRemoved()), this, SLOT(slotScratchPadItemDeleted()));
-    connect(categoryView, SIGNAL(lastItemRemoved()), this, SLOT(slotLastScratchPadItemDeleted()));
+    connect(categoryView, &WidgetBoxCategoryListView::scratchPadChanged,
+            this, &WidgetBoxTreeWidget::slotSave);
+    connect(categoryView, &WidgetBoxCategoryListView::pressed,
+            this, &WidgetBoxTreeWidget::pressed);
+    connect(categoryView, &WidgetBoxCategoryListView::itemRemoved,
+            this, &WidgetBoxTreeWidget::slotScratchPadItemDeleted);
+    connect(categoryView, &WidgetBoxCategoryListView::lastItemRemoved,
+            this, &WidgetBoxTreeWidget::slotLastScratchPadItemDeleted);
     setItemWidget(embed_item, 0, categoryView);
     return categoryView;
 }
@@ -650,8 +651,6 @@ WidgetBoxTreeWidget::CategoryList WidgetBoxTreeWidget::loadCustomCategoryList()
             icon_name = iconPrefix;
             icon_name += pluginName;
             m_pluginIcons.insert(icon_name, icon);
-        } else {
-            icon_name = QLatin1String(qtLogoC);
         }
 
         cat.addWidget(Widget(displayName, dom_xml, icon_name, Widget::Custom));
@@ -813,7 +812,8 @@ void WidgetBoxTreeWidget::slotLastScratchPadItemDeleted()
         m_scratchPadDeleteTimer = new QTimer(this);
         m_scratchPadDeleteTimer->setSingleShot(true);
         m_scratchPadDeleteTimer->setInterval(0);
-        connect(m_scratchPadDeleteTimer, SIGNAL(timeout()), this, SLOT(deleteScratchpad()));
+        connect(m_scratchPadDeleteTimer, &QTimer::timeout,
+                this, &WidgetBoxTreeWidget::deleteScratchpad);
     }
     if (!m_scratchPadDeleteTimer->isActive())
         m_scratchPadDeleteTimer->start();
@@ -877,8 +877,8 @@ void WidgetBoxTreeWidget::contextMenuEvent(QContextMenuEvent *e)
                             && topLevelRole(item->parent()) ==  SCRATCHPAD_ITEM;
 
     QMenu menu;
-    menu.addAction(tr("Expand all"), this, SLOT(expandAll()));
-    menu.addAction(tr("Collapse all"), this, SLOT(collapseAll()));
+    menu.addAction(tr("Expand all"), this, &WidgetBoxTreeWidget::expandAll);
+    menu.addAction(tr("Collapse all"), this, &WidgetBoxTreeWidget::collapseAll);
     menu.addSeparator();
 
     QAction *listModeAction = menu.addAction(tr("List View"));
@@ -892,14 +892,16 @@ void WidgetBoxTreeWidget::contextMenuEvent(QContextMenuEvent *e)
         iconModeAction->setChecked(true);
     else
         listModeAction->setChecked(true);
-    connect(listModeAction, SIGNAL(triggered()), SLOT(slotListMode()));
-    connect(iconModeAction, SIGNAL(triggered()), SLOT(slotIconMode()));
+    connect(listModeAction, &QAction::triggered, this, &WidgetBoxTreeWidget::slotListMode);
+    connect(iconModeAction, &QAction::triggered, this, &WidgetBoxTreeWidget::slotIconMode);
 
     if (scratchpad_menu) {
         menu.addSeparator();
-        menu.addAction(tr("Remove"), itemWidget(item, 0), SLOT(removeCurrentItem()));
+        WidgetBoxCategoryListView *listView = qobject_cast<WidgetBoxCategoryListView *>(itemWidget(item, 0));
+        Q_ASSERT(listView);
+        menu.addAction(tr("Remove"), listView, &WidgetBoxCategoryListView::removeCurrentItem);
         if (!m_iconMode)
-            menu.addAction(tr("Edit name"), itemWidget(item, 0), SLOT(editCurrentItem()));
+            menu.addAction(tr("Edit name"), listView, &WidgetBoxCategoryListView::editCurrentItem);
     }
     e->accept();
     menu.exec(mapToGlobal(e->pos()));
