@@ -233,7 +233,7 @@ private:
         Tok_Eof, Tok_class, Tok_friend, Tok_namespace, Tok_using, Tok_return,
         Tok_Q_OBJECT, Tok_Access, Tok_Cancel,
         Tok_Ident, Tok_String, Tok_Arrow, Tok_Colon, Tok_ColonColon,
-        Tok_Equals, Tok_LeftBracket, Tok_RightBracket,
+        Tok_Equals, Tok_LeftBracket, Tok_RightBracket, Tok_QuestionMark,
         Tok_LeftBrace, Tok_RightBrace, Tok_LeftParen, Tok_RightParen, Tok_Comma, Tok_Semicolon,
         Tok_Null, Tok_Integer,
         Tok_QuotedInclude, Tok_AngledInclude,
@@ -456,8 +456,12 @@ int CppParser::getChar()
 
 STRING(Q_OBJECT);
 STRING(class);
+STRING(final);
 STRING(friend);
 STRING(namespace);
+STRING(nullptr);
+STRING(Q_NULLPTR);
+STRING(NULL);
 STRING(operator);
 STRING(return);
 STRING(struct);
@@ -562,10 +566,9 @@ CppParser::TokenType CppParser::getToken()
                     // include
                     do {
                         yyCh = getChar();
-                    } while (yyCh != EOF && !isspace(yyCh));
-                    do {
+                    } while (yyCh != EOF && !isspace(yyCh) && yyCh != '"' && yyCh != '<' );
+                    while (isspace(yyCh))
                         yyCh = getChar();
-                    } while (isspace(yyCh));
                     int tChar;
                     if (yyCh == '"')
                         tChar = '"';
@@ -680,7 +683,13 @@ CppParser::TokenType CppParser::getToken()
             //qDebug() << "IDENT: " << yyWord;
 
             switch (yyWord.unicode()[0].unicode()) {
+            case 'N':
+                if (yyWord == strNULL)
+                    return Tok_Null;
+                break;
             case 'Q':
+                if (yyWord == strQ_NULLPTR)
+                    return Tok_Null;
                 if (yyWord == strQ_OBJECT)
                     return Tok_Q_OBJECT;
                 if (yyWord == strQ_SLOTS || yyWord == strQ_SIGNALS)
@@ -697,6 +706,8 @@ CppParser::TokenType CppParser::getToken()
             case 'n':
                 if (yyWord == strnamespace)
                     return Tok_namespace;
+                if (yyWord == strnullptr)
+                    return Tok_Null;
                 break;
             case 'o':
                 if (yyWord == stroperator) {
@@ -838,7 +849,7 @@ CppParser::TokenType CppParser::getToken()
 
                 forever {
                     if (yyCh == EOF || yyCh == '\n') {
-                        yyMsg() << "Unterminated C++ character\n";
+                        yyMsg() << qPrintable(LU::tr("Unterminated C++ character\n"));
                         break;
                     }
                     yyCh = getChar();
@@ -903,6 +914,9 @@ CppParser::TokenType CppParser::getToken()
             case ';':
                 yyCh = getChar();
                 return Tok_Semicolon;
+            case '?':
+                yyCh = getChar();
+                return Tok_QuestionMark;
             case '0':
                 yyCh = getChar();
                 if (yyCh == 'x') {
@@ -1484,13 +1498,17 @@ QString CppParser::transcode(const QString &str)
             if (c == '\n')
                 continue;
 
-            if (c == 'x') {
+            if (c == 'x' || c == 'u' || c == 'U') {
+                const bool unicode = (c != 'x');
                 QByteArray hex;
                 while (i < in.length() && isxdigit((c = in[i]))) {
                     hex += c;
                     i++;
                 }
-                out += hex.toUInt(0, 16);
+                if (unicode)
+                    out += QString(QChar(hex.toUInt(Q_NULLPTR, 16))).toUtf8();
+                else
+                    out += hex.toUInt(Q_NULLPTR, 16);
             } else if (c >= '0' && c < '8') {
                 QByteArray oct;
                 int n = 0;
@@ -1742,6 +1760,7 @@ void CppParser::parseInternal(ConversionData &cd, const QStringList &includeStac
     QString functionName;
 #endif
     bool yyTokColonSeen = false; // Start of c'tor's initializer list
+    bool yyTokIdentSeen = false; // Start of initializer (member or base class)
     metaExpected = true;
 
     prospectiveContext.clear();
@@ -1805,38 +1824,43 @@ void CppParser::parseInternal(ConversionData &cd, const QStringList &includeStac
             if (yyBraceDepth == namespaceDepths.count() && yyParenDepth == 0) {
                 NamespaceList quali;
                 HashString fct;
-                do {
-                    /*
-                      This code should execute only once, but we play
-                      safe with impure definitions such as
-                      'class Q_EXPORT QMessageBox', in which case
-                      'QMessageBox' is the class name, not 'Q_EXPORT'.
-                    */
-                    text = yyWord;
-                    text.detach();
-                    fct.setValue(text);
-                    yyTok = getToken();
-                } while (yyTok == Tok_Ident);
-                while (yyTok == Tok_ColonColon) {
-                    yyTok = getToken();
-                    if (yyTok != Tok_Ident)
-                        break; // Oops ...
-                    quali << fct;
+
+                // Find class name including qualification
+                forever {
                     text = yyWord;
                     text.detach();
                     fct.setValue(text);
                     yyTok = getToken();
+
+                    if (yyTok == Tok_ColonColon) {
+                        quali << fct;
+                        yyTok = getToken();
+                    } else if (yyTok == Tok_Ident) {
+                        if (yyWord == strfinal) {
+                            // C++11: final may appear immediately after the name of the class
+                            yyTok = getToken();
+                            break;
+                        }
+
+                        // Handle impure definitions such as 'class Q_EXPORT QMessageBox', in
+                        // which case 'QMessageBox' is the class name, not 'Q_EXPORT', by
+                        // abandoning any qualification collected so far.
+                        quali.clear();
+                    } else {
+                        break;
+                    }
                 }
-                if (yyTok == Tok_Colon) {
-                    // Skip any token until '{' since we might do things wrong if we find
-                    // a '::' token here.
+
+                if (yyTok == Tok_Colon || yyTok == Tok_Other) {
+                    // Skip any token until '{' or ';' since we might do things wrong if we find
+                    // a '::' or ':' token here.
                     do {
                         yyTok = getToken();
                         if (yyTok == Tok_Eof)
                             goto goteof;
                         if (yyTok == Tok_Cancel)
                             goto case_default;
-                    } while (yyTok != Tok_LeftBrace);
+                    } while (yyTok != Tok_LeftBrace && yyTok != Tok_Semicolon);
                 } else {
                     if (yyTok != Tok_LeftBrace) {
                         // Obviously a forward declaration. We skip those, as they
@@ -1849,7 +1873,7 @@ void CppParser::parseInternal(ConversionData &cd, const QStringList &includeStac
                     // Forward-declared class definitions can be namespaced.
                     NamespaceList nsl;
                     if (!fullyQualify(namespaces, quali, true, &nsl, 0)) {
-                        yyMsg() << "Ignoring definition of undeclared qualified class\n";
+                        yyMsg() << qPrintable(LU::tr("Ignoring definition of undeclared qualified class\n"));
                         break;
                     }
                     namespaceDepths.push(namespaces.count());
@@ -1958,6 +1982,11 @@ void CppParser::parseInternal(ConversionData &cd, const QStringList &includeStac
             yyTok = getToken();
             break;
         case Tok_Ident:
+            if (yyTokColonSeen &&
+                yyBraceDepth == namespaceDepths.count() && yyParenDepth == 0) {
+                // member or base class identifier
+                yyTokIdentSeen = true;
+            }
             yyTok = getToken();
             if (yyTok == Tok_LeftParen) {
                 switch (trFunctionAliasManager.trFunctionByName(yyWord)) {
@@ -2014,6 +2043,11 @@ void CppParser::parseInternal(ConversionData &cd, const QStringList &includeStac
             }
             break;
         case Tok_ColonColon:
+            if (yyTokIdentSeen) {
+                // member or base class identifier
+                yyTok = getToken();
+                break;
+            }
             if (yyBraceDepth == namespaceDepths.count() && yyParenDepth == 0 && !yyTokColonSeen)
                 prospectiveContext = prefix;
             prefix += strColons;
@@ -2026,16 +2060,20 @@ void CppParser::parseInternal(ConversionData &cd, const QStringList &includeStac
 #endif
             break;
         case Tok_RightBrace:
-            if (yyBraceDepth + 1 == namespaceDepths.count()) // class or namespace
-                truncateNamespaces(&namespaces, namespaceDepths.pop());
-            if (yyBraceDepth == namespaceDepths.count()) {
-                // function, class or namespace
-                if (!yyBraceDepth && !directInclude)
-                    truncateNamespaces(&functionContext, 1);
-                else
-                    functionContext = namespaces;
-                functionContextUnresolved.clear();
-                pendingContext.clear();
+            if (!yyTokColonSeen) {
+                if (yyBraceDepth + 1 == namespaceDepths.count()) {
+                    // class or namespace
+                    truncateNamespaces(&namespaces, namespaceDepths.pop());
+                }
+                if (yyBraceDepth == namespaceDepths.count()) {
+                    // function, class or namespace
+                    if (!yyBraceDepth && !directInclude)
+                        truncateNamespaces(&functionContext, 1);
+                    else
+                        functionContext = namespaces;
+                    functionContextUnresolved.clear();
+                    pendingContext.clear();
+                }
             }
             // fallthrough
         case Tok_Semicolon:
@@ -2074,15 +2112,22 @@ void CppParser::parseInternal(ConversionData &cd, const QStringList &includeStac
             yyTok = getToken();
             break;
         case Tok_LeftBrace:
-            if (!prospectiveContext.isEmpty()
-                && yyBraceDepth == namespaceDepths.count() + 1 && yyParenDepth == 0) {
-                pendingContext = prospectiveContext;
-                prospectiveContext.clear();
+            if (yyBraceDepth == namespaceDepths.count() + 1 && yyParenDepth == 0) {
+                if (!prospectiveContext.isEmpty()) {
+                    pendingContext = prospectiveContext;
+                    prospectiveContext.clear();
+                }
+                if (!yyTokIdentSeen) {
+                    // Function body
+                    yyTokColonSeen = false;
+                }
             }
-            yyTokColonSeen = false;
+            // fallthrough
+        case Tok_LeftParen:
+            yyTokIdentSeen = false;
             // fallthrough
         case Tok_Comma:
-        case Tok_LeftParen:
+        case Tok_QuestionMark:
             metaExpected = true;
             yyTok = getToken();
             break;
