@@ -64,9 +64,11 @@
 
 #ifdef Q_OS_WIN32
 #define QT_POPEN _popen
+#define QT_POPEN_READ "rb"
 #define QT_PCLOSE _pclose
 #else
 #define QT_POPEN popen
+#define QT_POPEN_READ "r"
 #define QT_PCLOSE pclose
 #endif
 
@@ -74,27 +76,8 @@ QT_BEGIN_NAMESPACE
 
 #define fL1S(s) QString::fromLatin1(s)
 
-namespace { // MSVC doesn't seem to know the semantics of "static" ...
-
-static struct {
-    QRegExp reg_variableName;
-} statics;
-
-}
-
-static void initStatics()
-{
-    if (!statics.reg_variableName.isEmpty())
-        return;
-
-    statics.reg_variableName.setPattern(QLatin1String("\\$\\(.*\\)"));
-    statics.reg_variableName.setMinimal(true);
-}
-
 QMakeGlobals::QMakeGlobals()
 {
-    initStatics();
-
     do_cache = true;
 
 #ifdef PROEVALUATOR_DEBUG
@@ -197,11 +180,11 @@ QMakeGlobals::ArgumentReturn QMakeGlobals::addCommandLineArguments(
 void QMakeGlobals::commitCommandLineArguments(QMakeCmdLineParserState &state)
 {
     if (!state.preconfigs.isEmpty())
-        state.precmds << (fL1S("CONFIG += ") + state.preconfigs.join(fL1S(" ")));
-    precmds = state.precmds.join(fL1S("\n"));
+        state.precmds << (fL1S("CONFIG += ") + state.preconfigs.join(QLatin1Char(' ')));
+    precmds = state.precmds.join(QLatin1Char('\n'));
     if (!state.postconfigs.isEmpty())
-        state.postcmds << (fL1S("CONFIG += ") + state.postconfigs.join(fL1S(" ")));
-    postcmds = state.postcmds.join(fL1S("\n"));
+        state.postcmds << (fL1S("CONFIG += ") + state.postconfigs.join(QLatin1Char(' ')));
+    postcmds = state.postcmds.join(QLatin1Char('\n'));
 
     if (xqmakespec.isEmpty())
         xqmakespec = qmakespec;
@@ -292,11 +275,24 @@ QStringList QMakeGlobals::getPathListEnv(const QString &var) const
 QString QMakeGlobals::expandEnvVars(const QString &str) const
 {
     QString string = str;
-    int rep;
-    QRegExp reg_variableName = statics.reg_variableName; // Copy for thread safety
-    while ((rep = reg_variableName.indexIn(string)) != -1)
-        string.replace(rep, reg_variableName.matchedLength(),
-                       getEnv(string.mid(rep + 2, reg_variableName.matchedLength() - 3)));
+    int startIndex = 0;
+    forever {
+        startIndex = string.indexOf(QLatin1Char('$'), startIndex);
+        if (startIndex < 0)
+            break;
+        if (string.length() < startIndex + 3)
+            break;
+        if (string.at(startIndex + 1) != QLatin1Char('(')) {
+            startIndex++;
+            continue;
+        }
+        int endIndex = string.indexOf(QLatin1Char(')'), startIndex + 2);
+        if (endIndex < 0)
+            break;
+        QString value = getEnv(string.mid(startIndex + 2, endIndex - startIndex - 2));
+        string.replace(startIndex, endIndex - startIndex + 1, value);
+        startIndex += value.length();
+    }
     return string;
 }
 
@@ -313,7 +309,7 @@ bool QMakeGlobals::initProperties()
     data = proc.readAll();
 #else
     if (FILE *proc = QT_POPEN(QString(QMakeInternal::IoUtils::shellQuote(qmake_abslocation)
-                                      + QLatin1String(" -query")).toLocal8Bit(), "r")) {
+                                      + QLatin1String(" -query")).toLocal8Bit(), QT_POPEN_READ)) {
         char buff[1024];
         while (!feof(proc))
             data.append(buff, int(fread(buff, 1, 1023, proc)));
