@@ -51,14 +51,20 @@ int main(int argc, char **argv)
     QCoreApplication::setOrganizationDomain(QStringLiteral("qt-project.org"));
 
     QCommandLineParser commandLineParser;
+    const QCommandLineOption noGlOption(QStringLiteral("no-gl"), QStringLiteral("Do not output GL information"));
     const QCommandLineOption glExtensionOption(QStringLiteral("gl-extensions"), QStringLiteral("List GL extensions"));
+    const QCommandLineOption fontOption(QStringLiteral("fonts"), QStringLiteral("Output list of fonts"));
     commandLineParser.setApplicationDescription(QStringLiteral("Prints diagnostic output about the Qt library."));
+    commandLineParser.addOption(noGlOption);
     commandLineParser.addOption(glExtensionOption);
+    commandLineParser.addOption(fontOption);
     commandLineParser.addHelpOption();
     commandLineParser.process(app);
-    unsigned flags = 0;
+    unsigned flags = commandLineParser.isSet(noGlOption) ? 0u : unsigned(QtDiagGl);
     if (commandLineParser.isSet(glExtensionOption))
         flags |= QtDiagGlExtensions;
+    if (commandLineParser.isSet(fontOption))
+        flags |= QtDiagFonts;
 
     std::wcout << qtDiag(flags).toStdWString();
     return 0;
