@@ -103,7 +103,8 @@ enum QtModule
     Qt3DInputModule           = 0x0000200000000000,
     QtLocationModule          = 0x0000400000000000,
     QtWebChannelModule        = 0x0000800000000000,
-    QtTextToSpeechModule      = 0x0001000000000000
+    QtTextToSpeechModule      = 0x0001000000000000,
+    QtSerialBusModule         = 0x0002000000000000
 };
 
 struct QtModuleEntry {
@@ -161,7 +162,8 @@ static QtModuleEntry qtModuleEntries[] = {
     { Qt3DInputModule, "3dinput", "Qt53DInput", 0 },
     { QtLocationModule, "geoservices", "Qt5Location", 0 },
     { QtWebChannelModule, "webchannel", "Qt5WebChannel", 0 },
-    { QtTextToSpeechModule, "texttospeech", "Qt5TextToSpeech", 0 }
+    { QtTextToSpeechModule, "texttospeech", "Qt5TextToSpeech", 0 },
+    { QtSerialBusModule, "serialbus", "Qt5SerialBus", 0 }
 };
 
 static const char webKitProcessC[] = "QtWebProcess";
@@ -818,6 +820,8 @@ static inline quint64 qtModuleForPlugin(const QString &subDirName)
         return Qt3DRendererModule;
     if (subDirName == QLatin1String("texttospeech"))
         return QtTextToSpeechModule;
+    if (subDirName == QLatin1String("canbus"))
+        return QtSerialBusModule;
     return 0; // "designer"
 }
 
