@@ -224,8 +224,14 @@ AppxPhoneEngine::AppxPhoneEngine(Runner *runner)
     d->hasFatalError = true;
 
     ComPtr<IStream> manifestStream;
-    HRESULT hr = SHCreateStreamOnFile(wchar(d->manifest), STGM_READ, &manifestStream);
-    RETURN_VOID_IF_FAILED("Failed to open manifest stream");
+    HRESULT hr;
+    if (d->manifestReader) {
+        hr = d->manifestReader->GetStream(&manifestStream);
+        RETURN_VOID_IF_FAILED("Failed to query manifest stream from manifest reader.");
+    } else {
+        hr = SHCreateStreamOnFile(wchar(d->manifest), STGM_READ, &manifestStream);
+        RETURN_VOID_IF_FAILED("Failed to open manifest stream");
+    }
 
     if (!getPhoneProductId(manifestStream.Get(), &d->productId)) {
         qCWarning(lcWinRtRunner) << "Failed to read phone product ID from the manifest.";
@@ -258,6 +264,14 @@ AppxPhoneEngine::~AppxPhoneEngine()
 
 QString AppxPhoneEngine::extensionSdkPath() const
 {
+#if _MSC_VER >= 1900
+    const QByteArray extensionSdkDirRaw = qgetenv("ExtensionSdkDir");
+    if (extensionSdkDirRaw.isEmpty()) {
+        qCWarning(lcWinRtRunner) << "The environment variable ExtensionSdkDir is not set.";
+        return QString();
+    }
+    return QString::fromLocal8Bit(extensionSdkDirRaw);
+#else // _MSC_VER < 1900
     HKEY regKey;
     LONG hr = RegOpenKeyEx(
                 HKEY_LOCAL_MACHINE,
@@ -278,6 +292,7 @@ QString AppxPhoneEngine::extensionSdkPath() const
 
     return QString::fromWCharArray(pathData, (pathLength - 1) / sizeof(wchar_t))
             + QLatin1String("ExtensionSDKs");
+#endif // _MSC_VER < 1900
 }
 
 bool AppxPhoneEngine::installPackage(IAppxManifestReader *reader, const QString &filePath)
@@ -323,7 +338,12 @@ bool AppxPhoneEngine::installPackage(IAppxManifestReader *reader, const QString
     _bstr_t packagePath(wchar(QDir::toNativeSeparators(filePath)));
     hr = connection->InstallApplication(productId, productId, deploymentFlagsAsGenre,
                                         packageTypeAsIconPath, packagePath);
-    RETURN_FALSE_IF_FAILED("Failed to install the package");
+    if (hr == 0x80073d06) { // No public E_* macro available
+        qCWarning(lcWinRtRunner) << "Found a newer version of " << filePath
+                                 << " on the target device, skipping...";
+    } else {
+        RETURN_FALSE_IF_FAILED("Failed to install the package");
+    }
 
     return true;
 }
@@ -379,20 +399,22 @@ bool AppxPhoneEngine::install(bool removeFirst)
     if (!installDependencies())
         return false;
 
-    const QDir base = QFileInfo(d->manifest).absoluteDir();
-    const QString packageFileName = base.absoluteFilePath(d->packageFamilyName + QStringLiteral(".appx"));
-    if (!createPackage(packageFileName))
-        return false;
+    const QDir base = QFileInfo(d->executable).absoluteDir();
+    const bool existingPackage = d->runner->app().endsWith(QLatin1String(".appx"));
+    const QString packageFileName = existingPackage
+                                      ? d->runner->app()
+                                      : base.absoluteFilePath(d->packageFamilyName + QStringLiteral(".appx"));
+    if (!existingPackage) {
+        if (!createPackage(packageFileName))
+            return false;
 
-    ComPtr<IStream> manifestStream;
-    hr = SHCreateStreamOnFile(wchar(d->manifest), STGM_READ, &manifestStream);
-    RETURN_FALSE_IF_FAILED("Failed to open manifest stream");
+        if (!sign(packageFileName))
+            return false;
+    } else {
+        qCDebug(lcWinRtRunner) << "Installing existing package.";
+    }
 
-    ComPtr<IAppxManifestReader> manifestReader;
-    hr = d->packageFactory->CreateManifestReader(manifestStream.Get(), &manifestReader);
-    RETURN_FALSE_IF_FAILED("Failed to create manifest reader for installation");
-
-    return installPackage(manifestReader.Get(), packageFileName);
+    return installPackage(d->manifestReader.Get(), packageFileName);
 }
 
 bool AppxPhoneEngine::remove()
@@ -488,13 +510,13 @@ bool AppxPhoneEngine::waitForFinished(int secs)
 
 bool AppxPhoneEngine::stop()
 {
-    Q_D(AppxPhoneEngine);
     qCDebug(lcWinRtRunner) << __FUNCTION__;
 
     if (!connect())
         return false;
 
 #if 0 // This does not actually stop the app - QTBUG-41946
+    Q_D(AppxPhoneEngine);
     ComPtr<ICcConnection3> connection;
     HRESULT hr = d->connection.As(&connection);
     RETURN_FALSE_IF_FAILED("Failed to cast connection object");
